Program DemoDS5T;
{
;===================================================================;
;                                                                   ;
;     Program:     DemoDS5T                                         ;
;                                                                   ;
;     Version:     1.1                                              ;
;                                                                   ;
;     Date:        June 7, 1988                                     ;
;                                                                   ;
;     Programmer:  Stephen M. Curry                                 ;
;                                                                   ;
;     Computers:   ACS PC running DOS 3.2 and DS5000T               ;
;                                                                   ;
;     Compiler:    Borland Turbo Pascal Version 4.0                 ;
;                                                                   ;
;     Purpose:     This is a program to test the performance of     ;
;                  a DS5000T by relaying date/time information      ;
;                  between it and the PC and comparing it against   ;
;                  the PC clock.  This program requires that        ;
;                  DEMODS5T.HEX be installed in the DS5000T, and    ;
;                  that the DS5000T be installed in a DS9060 RS232  ;
;                  Fixture with its serial selection switch in the  ;
;                  B position and connected to one of the COM       ;
;                  ports of the PC.                                 ;
;                                                                   ;
;===================================================================;
}

Uses DOS, CRT;

Const
   BR = 9600;  { Baud Rate }

Type
   DateTime = Array[0..7] of Byte;

{
   The following procedure is used to make the cursor disappear or
   reappear from the screen in order to provide a less cluttered
   refresh display.  Cursor(False) saves the current appearence of
   the cursor and then makes the cursor disappear.  Cursor(True)
   restores the original appearence of the cursor.
}
Var  SaveCursor : Integer;
Procedure Cursor(B: Boolean);
Var Regs : Registers;
Begin
   With Regs do Begin
      If B then Begin
         AX := $100;
         CX := SaveCursor;
      End else Begin
         AX := $300;
         Intr($10, Regs);
         SaveCursor := CX;
         AX := $100;
         CX := $F00;
      End;
      Intr($10, Regs);
   End;
End;

{
   The following procedure returns the date/time information
   from the PC clock in the array X.
}
Procedure Timer(Var X: DateTime);
Var
   I   : Byte;
   Regs: Registers;
Begin
   with Regs do Begin
      AX := $2C00;
      MSDos(Regs);
      X[3] := CH;
      X[2] := CL;
      X[1] := DH;
      X[0] := DL;
      AX := $2A00;
      MSDos(Regs);
      X[7] := CX - 1900;
      X[6] := DH;
      X[5] := DL;
      X[4] := Succ(AL);
   End;
   For I := 0 to 7 do X[I] := X[I] div 10 shl 4 + X[I] mod 10;
End;

{
   The following function returns a hexadecimal representation
   of the byte variable V.
}
Type HexTyp = String[2];
Function Hex(V: Byte): HexTyp;
Var
   I: Byte;
   S: HexTyp;
Begin
   S := '';
   While V <> 0 do Begin
      I := V and $F;  V := V shr 4;
      If I > 9 then I := I + 7;
      S := Char(I + $30) + S;
   End;
   While Length(S) < 2 do S := '0' + S;
   Hex := S;
End;

{
   The following function returns the signed integer difference
   between the two BCD coded numbers A and B.  (Diff = "A - B")
}
Function Diff(A, B: Byte): Integer;
Var X, Y : Byte;
Begin
   X := 10 * (A shr 4) + (A and $F);
   Y := 10 * (B shr 4) + (B and $F);
   Diff := X - Y;
End;

{
   The following variables are used in the main program.
}
Var
   PA : Array[0..3] of Integer Absolute $40:0;  { Array of port addresses }
   DST, PCT : DateTime;                               { Time data buffers }
   NP, N, I, J, X1, X2, X3, Y : Byte;
   P, BT, K : Integer;
   CH : Char;
   Ticks : LongInt Absolute $40:$6C;
   ST    : LongInt;
{
   The main program begins here.
}
Begin
   DirectVideo := False;
   TextBackground(0);
   ClrScr; Window(11, 4, 69, 22);
   TextColor(14); TextBackground(1);
   ClrScr; Window(13, 5, 69, 22);
   BT := Round(115200.0 / BR);   { Calculate the baud timer period }
   NP := 0;
   While NP < 4 do Begin  { Look through the COM ports for DS5000T }
      P := PA[NP];
      NP := Succ(NP);
      If P = 0 then Exit;
      Port[P + 3] := $83;
      Port[P + 0] := Lo(BT);  { Set timer for              }
      Port[P + 1] := Hi(BT);  { Baud rate generator        }
      Port[P + 3] := $03;     { 8 data, 1 stop, no parity  }
      Port[P + 1] := $00;     { No interrupts              }
      {
         Clear DTR signal to start the program in the DS5000T
      }
      Port[P + 4] := $02;     { Clear DTR signal (RTS set) }
      Delay(1000);
      N := 0;
      {
         Send out ten U's and look for ten V's returned.  This
         confirms that the DS5000T is present and contains the
         RELAY program.
      }
      For I := 1 to 10 do Begin
         Repeat until Port[P + 5] and $20 = $20;
         Port[P] := Byte('U');
         ST := Ticks;
         Repeat Until (Port[P + 5] and 1 = 1) or (Ticks - ST > 1);
         If Port[P + 5] and 1 = 1 then If Port[P] = Byte('V') then N := Succ(N);
      End;
      If N = 10 then Begin
         WriteLn('DS5000T found on COM',NP); WriteLn;
         Write('Do you want to set it now (Y/N) ? ');
         CH := ReadKey; WriteLn(CH);
         If (CH = 'Y') or (CH = 'y') then Begin
            Repeat until Port[P + 5] and $20 = $20;
            Port[P] := Byte('W'); Delay(100);
            Timer(PCT);     { Read the time from the PC's clock }
            For J := 0 to 7 do Begin
               Repeat until Port[P + 5] and $20 = $20;
               Port[P] := PCT[J];  { Send PC time to DS5000T }
            End;
            WriteLn; WriteLn('PC Date/Time sent to DS5000T');
         End else WriteLn;
         WriteLn;
         Write('Date/Time Information   ');
         X1 := WhereX;
         Write('DS5000T   ');
         X2 := WhereX;
         Write('PC Clock   ');
         X3 := WhereX;
         Write('Difference');
         WriteLn; WriteLn;
         Y := WhereY;
         WriteLn('Seconds');
         WriteLn('Minutes');
         WriteLn('Hours');
         WriteLn('Day of Week');
         WriteLn('Day of Month');
         WriteLn('Month');
         WriteLn('Year');
         WriteLn;
         WriteLn('Press any key to end demonstration program');
         Cursor(False);  { Remove cursor to clean up the refresh }
         Repeat
            Timer(PCT);         { Read the time from the PC clock }
            Repeat until Port[P + 5] and $20 = $20;
            Port[P] := Byte('R');
            For J := 0 to 7 do Begin
               ST := Ticks;
               Repeat until (Port[P + 5] and $1 = $1) or (Ticks - ST > 1);
               DST[J] := Port[P];   { Read time from the DS5000T }
            End;
            {
               Print out the time from the DS5000T, the time from
               the PC clock, and the difference between them.
            }
            For J := 1 to 7 do Begin
               K := Y + J - 1;
               GotoXY(X1, K); Write(Hex(DST[J]), ' ');
               GotoXY(X2, K); Write(Hex(PCT[J]), ' ');
               GotoXY(X3, K); WriteLn(Diff(DST[J], PCT[J]), '   ');
            End;
         Until KeyPressed;
         CH := ReadKey;
         Cursor(True);  { Restore the original cursor appearence }
         Window(1, 1, 79, 25); ClrScr;
         Exit;
      End;
   End;
End.
